using Vector2 = Godot.Vector2;
using System;
using System.Collections;

public class Player {
	private readonly VirtualBoard board;
	private readonly SelectedPiece selectedNode;
	
	public EventHandler<VirtualBoard.MoveEventArgs> OnMove;
	public bool CanMove { get; set; } = true;

	public enum PlayerType {
		Human,
		AI
	}

	public Player(VirtualBoard board, PlayerType type = PlayerType.Human)
	{
		this.board = board;
		this.selectedNode = new SelectedPiece(board);
	}

	public void HandleBoardPosClick(Vector2 clickPos) {
		if (board.ArrPosOutOfRange(clickPos)) return;
		// GD.Print($"VirtualBoard {clickPos} clicked");
		VirtualPiece clickChess = board.GetPiece(clickPos);

		if (!selectedNode.HasSelected()) {
			// Select piece
			if (clickChess == null) {
				// selectedNode.Clear();
				return;
			}
			selectedNode.SetPos(clickPos);
		} else if (clickChess == selectedNode.GetPiece()) {
			// Unselect piece
			selectedNode.Clear();
		} else {
			// Move piece
			// GD.Print("default MoveFunc Move: ", selectedNode.GetPos(), "->", clickPos);
			if (CanMove) MoveAndRecord(clickPos, selectedNode.GetPos());
		}
	}

	public void MoveAndRecord(Vector2 toPos, Vector2 fromPos) {
		// GD.Print($"{fromPos} move to {toPos}");
		VirtualPiece toChess = board.GetPiece(toPos);
        VirtualPiece fromChess = board.GetPiece(fromPos);
		fromChess?.Selected(false);
		
		// MUST BE THERE !!! 防止删除节点后在启动回调导致错误
		OnMove?.Invoke(this, new VirtualBoard.MoveEventArgs { From = fromPos, To = toPos });

		VirtualPiece NowNode;
        if (toChess != null) {
			NowNode = toChess;
            board.RemovePiece(toPos);
        } else {
			NowNode = toChess;
		}
        board.MovePiece(fromPos, toPos);

		selectedNode.Clear();
	}

	public void SelectedClear() {
		selectedNode.Clear();
	}

	public void SetAllowedPieces(ArrayList allowedPieces) {
		selectedNode.allowedPieces = allowedPieces;
	}

	private class SelectedPiece {
		// Called when the node enters the scene tree for the first time.
		private Vector2 selectedNodePos = Vector2.Inf;
		private VirtualPiece piece;
		private readonly VirtualBoard board;
		public ArrayList allowedPieces = null;

		public SelectedPiece(VirtualBoard board) {
			this.board = board;
		}

		public void Clear() {
			if (selectedNodePos != Vector2.Inf) {
				selectedNodePos = Vector2.Inf;
				piece.Selected(false);
			}
		}

		public void SetPos(Vector2 pos) {
			// piece = board.GetNodeFromBoard(pos) as VirtualPiece;
			piece = board.GetPiece(pos);
			if (allowedPieces != null && allowedPieces.Contains(piece) == false) {
				return;
			}
			selectedNodePos = pos;
			piece.Selected(true);
		}

		public VirtualPiece GetPiece() {
			return piece;
		}

		public Vector2 GetPos() {
			return selectedNodePos;
		}

		public bool HasSelected() {
			return selectedNodePos != Vector2.Inf;
		}
	}
}
